@@ -7,7 +7,7 @@
  * Written by Chad Trabant, ORFEUS/EC-Project MEREDIAN
  * The liss2ew sources were used as a template, thanks Pete Lombard.
  *
- * modified 2006.066
+ * modified 2010.335
  ***************************************************************************/
 
 /* System includes */
@@ -25,6 +25,25 @@
 #include <libslink.h>
 #include "slink2ew.h"
 
+#ifdef _WINNT
+#include <windows.h>
+
+struct timespec {
+ long    tv_sec;         /* seconds since Jan. 1, 1970 - not used by nanosleep in this version*/
+ long    tv_nsec;        /* and nanoseconds */
+};
+ 
+void nanosleep(struct timespec *tv, struct timespec *ta)
+{
+	unsigned long dword;
+	dword=tv->tv_nsec/1000000;
+	if (dword == 0) 
+	{
+	  dword=1; /* windows Sleep()  at 0 just returns, and we probably want one msec of sleep at least to minimize spinning */
+	}
+	Sleep(dword);
+}
+#endif /* WIN_NT */
 
 /***************************************************************************
  *
@@ -42,10 +61,12 @@ pid_t         myPid;             /* Process ID                           */
 char  ringName[MAXRINGNAMELEN];  /* Name of destination ring for data    */
 long  ringKey;                   /* Key to output shared memory region   */
 char  myModName[MAXMODNAMELEN];  /* Name of module instance              */
+long  maxrate = 0;               /* Max rate in bytes/sec, 0 to disable  */
 int   forcetracebuf = 0;         /* Switch to force TRACEBUF             */
 
 unsigned char myModId;           /* ID of this module                    */
 unsigned char myInstId;          /* Installation running this module     */
+unsigned char myInstIdConf = 0;  /* Installation ID configuration flag   */
 unsigned char typeError;         /* Error message type                   */
 unsigned char typeHeartbeat;     /* Heartbeat message type               */
 unsigned char typeWaveform;      /* Waveform message type TRACEBUF       */
@@ -70,6 +91,8 @@ main (int argc, char **argv)
   int packetcnt = 0;
   time_t timeNow;                /* current time                  */
   time_t timeLastBeat = 0;       /* time last heartbeat was sent  */
+  struct timeval timeLastPkt;    /* time last packet was received */
+  struct timeval timeThisPkt;    /* time this packet was received */
 
   int prethrottle = 0;
   int throttlecnt = 1;
@@ -93,6 +116,10 @@ main (int argc, char **argv)
   
   logit ("t", "slink2ew version %s\n", VERSION);
   
+  /* Initialize timing variable used in rate limit */
+  if ( maxrate > 0 )
+    gettimeofday (&timeLastPkt, NULL);
+  
   /* Loop with the connection manager */
   while ( (retval = sl_collect_nb (slconn, &slpack)) )
     {
@@ -130,6 +157,31 @@ main (int argc, char **argv)
 		  packetcnt = 0;
 		}
 	    }
+	  /* Limit processing speed to specified maximum data rate */
+	  /* warning: rudimentary trapping for overflow errors */
+	  if ( maxrate > 0 )
+	    {
+	      long nsec;
+	      
+	      gettimeofday (&timeThisPkt, NULL);
+	      
+	      /* Minimum nanosec for packet size and maxrate */
+	      nsec = (long) (SLRECSIZE * 1.0e9 / maxrate);
+	      
+	      /* Subtract interval since last packet */
+	      nsec -= ( 1.0e9 * (timeThisPkt.tv_sec - timeLastPkt.tv_sec) +
+			1.0e3 * (timeThisPkt.tv_usec - timeLastPkt.tv_usec) );
+	      
+	      if ( nsec > 0 && nsec < 1e9 )
+		{  /* Pad interval to limit rate */
+		  struct timespec naptime;
+		  naptime.tv_sec = 0;
+		  naptime.tv_nsec = nsec;
+		  nanosleep (&naptime, NULL);
+		}
+	      
+	      gettimeofday (&timeLastPkt, NULL);
+	    }
 	  
 	  prethrottle = 0;
 	}
@@ -190,13 +242,14 @@ packet_handler (char *msrecord, int packet_type,
   char *type[]  = { "Data", "Detection", "Calibration", "Timing",
 		    "Message", "General", "Request", "Info",
                     "Info (terminated)", "KeepAlive" };
-
+  
   /* Process waveform data */
   if ( packet_type == SLDATA )
     {
       sl_log (0, 1, "seq %d, Received %s blockette\n",
 	      seqnum, type[packet_type]);
-
+      
+      /* Send data to ring ignoring return value, continue regardless of errors */
       mseed2ewring (msrecord, &regionOut, &waveLogo);
     }
   else if ( packet_type == SLKEEP )
@@ -216,24 +269,39 @@ packet_handler (char *msrecord, int packet_type,
  * Unpack a Mini-SEED record, populate a TracePacket and send it to
  * the ring.
  *
+ * Return EW_SUCCESS on success and EW_FAILURE on errors.
  ***************************************************************************/
 int
 mseed2ewring (char *msrecord, SHM_INFO *pregionOut, MSG_LOGO *pwaveLogo)
 {
-  static MSrecord * msr = NULL;
+  static SLMSrecord * msr = NULL;
   TracePacket tbuf;
   int tracebuf2 = 0;   /* TRACEBUF2 => 0: none, 1: available, 2: populated */
   int len;
   uint32_t *samples;
   
-  msr_parse (slconn->log, msrecord, &msr, 1, 1);
+  /* Parse Mini-SEED record and unpack data samples */
+  sl_msr_parse (slconn->log, msrecord, &msr, 1, 1);
+  
+  /* Check for parsing/unpacking errors */
+  if ( ! msr )
+    {
+      logit("et", "slink2ew: Error - skipping bogus MSrecord.\n");
+      return EW_FAILURE;
+    }
+  else if ( msr->numsamples == -1 )
+    {
+      logit("et", "slink2ew: Error - skipping bogus MSrecord. numsamples = -1.\n");
+      sl_msr_free (&msr);
+      return EW_FAILURE;
+    }
   
   /* TRACE_HEADER and TRACE2_HEADER are the same size */
   memset (&tbuf, 0, sizeof(TRACE_HEADER));
   
   /* Log packet details if the verbosity is high */
   if ( verbose > 1 )
-    msr_print (slconn->log, msr, verbose - 1);
+    sl_msr_print (slconn->log, msr, verbose - 1);
   
   /* Create a TRACEBUF2 message if supported */
 #ifdef TRACE2_STA_LEN
@@ -244,24 +312,24 @@ mseed2ewring (char *msrecord, SHM_INFO *pregionOut, MSG_LOGO *pwaveLogo)
       tbuf.trh2.pinno = 0;
       tbuf.trh2.nsamp = msr->numsamples;
       
-      tbuf.trh2.starttime = msr_depochstime(msr);
-      msr_dsamprate (msr, &tbuf.trh2.samprate);
+      tbuf.trh2.starttime = sl_msr_depochstime(msr);
+      sl_msr_dsamprate (msr, &tbuf.trh2.samprate);
       tbuf.trh2.endtime = (tbuf.trh2.starttime +
 			   ((tbuf.trh2.nsamp - 1) / tbuf.trh2.samprate));
       
-      strncpclean(tbuf.trh2.net, msr->fsdh.network, 2);
-      strncpclean(tbuf.trh2.sta, msr->fsdh.station, 5);
-      strncpclean(tbuf.trh2.chan, msr->fsdh.channel, 3);
+      sl_strncpclean(tbuf.trh2.net, msr->fsdh.network, 2);
+      sl_strncpclean(tbuf.trh2.sta, msr->fsdh.station, 5);
+      sl_strncpclean(tbuf.trh2.chan, msr->fsdh.channel, 3);
       
       if ( strncmp(msr->fsdh.location, "  ", 2) == 0 )
-	strncpclean(tbuf.trh2.loc, LOC_NULL_STRING, 2);
+	sl_strncpclean(tbuf.trh2.loc, LOC_NULL_STRING, 2);
       else
-	strncpclean(tbuf.trh2.loc, msr->fsdh.location, 2);
+	sl_strncpclean(tbuf.trh2.loc, msr->fsdh.location, 2);
       
       tbuf.trh2.version[0] = TRACE2_VERSION0;
       tbuf.trh2.version[1] = TRACE2_VERSION1;
       
-      /* The decoding always produces 32-bit integers in host byte order */
+      /* The decoding produces 32-bit integers in host byte order */
 #ifdef _INTEL
       strcpy(tbuf.trh2.datatype, "i4");
 #endif
@@ -282,16 +350,16 @@ mseed2ewring (char *msrecord, SHM_INFO *pregionOut, MSG_LOGO *pwaveLogo)
       tbuf.trh.pinno = 0;
       tbuf.trh.nsamp = msr->numsamples;
       
-      tbuf.trh.starttime = msr_depochstime(msr);
-      msr_dsamprate (msr, &tbuf.trh.samprate);
+      tbuf.trh.starttime = sl_msr_depochstime(msr);
+      sl_msr_dsamprate (msr, &tbuf.trh.samprate);
       tbuf.trh.endtime = (tbuf.trh.starttime +
 			  ((tbuf.trh.nsamp - 1) / tbuf.trh.samprate));
       
-      strncpclean(tbuf.trh.net, msr->fsdh.network, 2);
-      strncpclean(tbuf.trh.sta, msr->fsdh.station, 5);
-      strncpclean(tbuf.trh.chan, msr->fsdh.channel, 3);
+      sl_strncpclean(tbuf.trh.net, msr->fsdh.network, 2);
+      sl_strncpclean(tbuf.trh.sta, msr->fsdh.station, 5);
+      sl_strncpclean(tbuf.trh.chan, msr->fsdh.channel, 3);
       
-      /* The decoding always produces 32-bit integers in host byte order */
+      /* The decoding produces 32-bit integers in host byte order */
 #ifdef _INTEL
       strcpy(tbuf.trh.datatype, "i4");
 #endif
@@ -353,13 +421,13 @@ report_status( MSG_LOGO * pLogo, short code, char * message )
   char          outMsg[MAXMESSAGELEN];  /* The outgoing message.        */
   time_t        msgTime;        /* Time of the message.                 */
 
-  /*  Get the time of the message                                       */
+  /*  Get the time of the message */
   time( &msgTime );
   
   /* Build & process the message based on the type */
   if ( pLogo->type == typeHeartbeat )
   {
-    sprintf( outMsg, "%ld %ld\n\0", (long) msgTime, (long) myPid );
+    sprintf( outMsg, "%ld %ld\n\0", (long) msgTime,(long) myPid );
     
     /* Write the message to the output region */
     if ( tport_putmsg( &regionOut, &hrtLogo, (long) strlen( outMsg ),
@@ -416,43 +484,45 @@ configure (char ** argvec)
   }
 
   /* Read node configuration info */
-  if ( GetLocalInst( &myInstId) != 0 )
+  if ( ! myInstIdConf )
   {
-    fprintf(stderr, "slink2ew: Error getting myInstId.\n" );
-    exit (EW_FAILURE);
+    if ( GetLocalInst( &myInstId) != 0 )
+    {
+      fprintf (stderr, "slink2ew: Error getting myInstId.\n" );
+      exit (EW_FAILURE);
+    }
+    myInstIdConf = 1;
   }
   
   /* Lookup the ring key */
   if ((ringKey = GetKey (ringName) ) == -1) 
   {
-    fprintf (stderr,
-	     "slink2ew:  Invalid ring name <%s>; exitting!\n", ringName);
+    fprintf (stderr, "slink2ew:  Invalid ring name <%s>; exitting!\n", ringName);
     exit (EW_FAILURE);
   }
   
   /* Look up message types of interest */
   if (GetType ("TYPE_HEARTBEAT", &typeHeartbeat) != 0) 
   {
-    fprintf (stderr, 
-             "slink2ew: Invalid message type <TYPE_HEARTBEAT>; exitting!\n");
+    fprintf (stderr, "slink2ew: Invalid message type <TYPE_HEARTBEAT>; exitting!\n");
     exit (EW_FAILURE);
   }
   if (GetType ("TYPE_ERROR", &typeError) != 0) 
   {
-    fprintf (stderr, 
-             "slink2ew: Invalid message type <TYPE_ERROR>; exitting!\n");
+    fprintf (stderr, "slink2ew: Invalid message type <TYPE_ERROR>; exitting!\n");
     exit (EW_FAILURE);
   }
 
   if (GetType ("TYPE_TRACEBUF", &typeWaveform) != 0) 
   {
-    fprintf (stderr, 
-             "slink2ew: Invalid message type <TYPE_TRACEBUF>; exitting!\n");
+    fprintf (stderr, "slink2ew: Invalid message type <TYPE_TRACEBUF>; exitting!\n");
     exit (EW_FAILURE);
   }
   
-  /* No GetType error checking as this type will not exist in all versions */
+  /* GetType for TRACEBUF2 only if supported */
+#ifdef TRACE2_STA_LEN
   GetType ("TYPE_TRACEBUF2", &typeWaveform2);
+#endif
   
   /* Set up logos for outgoing messages */
   hrtLogo.instid = myInstId;
@@ -564,17 +634,17 @@ proc_configfile (char * configfile)
 	{
           if (strlen(str) >= MAXMODNAMELEN)
           {
-            fprintf(stderr, "MyModId too long; max is %d\n", MAXMODNAMELEN -1);
+            fprintf(stderr, "MyModId too long; max is %d\n", MAXMODNAMELEN - 1);
             return EW_FAILURE;
           }
 
           strcpy (myModName, str);
-
+	  
 	  /* Lookup module ID */
 	  if ( GetModId( myModName, &myModId) != 0 )
 	  {
-	    fprintf( stderr, "slink2ew: Error getting myModId.\n" );
-	    exit (EW_FAILURE);
+	    fprintf( stderr, "slink2ew: Error getting module ID for %s\n", myModName);
+	    return EW_FAILURE;
 	  }
         }
       }
@@ -648,7 +718,7 @@ proc_configfile (char * configfile)
 	paramdir = getenv ( "EW_PARAMS" );
 	statefile = (char *) malloc (strlen(paramdir) + 16);
 
-        #ifdef _INTEL
+        #ifdef _WINNT
 	sprintf (statefile, "%s\\slink%d.state", paramdir, myModId);
         #else  /* *nix brand */
 	sprintf (statefile, "%s/slink%d.state", paramdir, myModId);
@@ -707,11 +777,33 @@ proc_configfile (char * configfile)
         }
       }
       
+      else if (k_its ("MaxRate"))
+      {
+	if ( (str = k_str ()) )
+	{
+	  maxrate = k_int();
+	}
+      }
+
       else if (k_its ("ForceTraceBuf1"))
       {
         forcetracebuf = k_int();
       }
-      
+
+      else if (k_its ("instId"))
+      {
+        if ( (str = k_str ()) )
+        {
+          if ( GetInst (str, &myInstId) )
+          {
+            fprintf(stderr, "slink2ew: Unknown installation ID '%s'\n", str);
+            return EW_FAILURE;
+          }
+
+	  myInstIdConf = 1;
+        }
+      }
+
       else if ( k_its ("Selectors"))
       {
         if ( (str = k_str ()) )
