@@ -7,7 +7,7 @@
  * 	Istituto Nazionale di Geofisica e Vulcanologia - Italy
  *	quintiliani@ingv.it
  *
- * $Id: nmxptool_getoptlong.c,v 1.9 2010/03/25 13:25:46 quintiliani Exp $
+ * $Id: nmxptool_getoptlong.c 4165 2011-01-24 15:19:28Z quintiliani $
  *
  */
 
@@ -23,7 +23,6 @@
 
 #include "nmxptool_getoptlong.h"
 
-
 const NMXPTOOL_PARAMS NMXPTOOL_PARAMS_DEFAULT =
 {
     NULL,
@@ -41,6 +40,7 @@ const NMXPTOOL_PARAMS NMXPTOOL_PARAMS_DEFAULT =
     DEFAULT_RATE,
     NULL,
     DEFAULT_DELAY,
+    DEFAULT_SPANINTERVAL,
     DEFAULT_MAX_TOLERABLE_LATENCY,
     DEFAULT_TIMEOUTRECV,
     DEFAULT_VERBOSE_LEVEL,
@@ -55,6 +55,9 @@ const NMXPTOOL_PARAMS NMXPTOOL_PARAMS_DEFAULT =
     DEFAULT_NETWORKDELAY,
     DEFAULT_LISTEN_PORT,
     DEFAULT_TIMING_QUALITY,
+    DEFAULT_QUALITY_INDICATOR,
+    DEFAULT_ENCODING,
+    DEFAULT_RECLEN_MINISEED,
     0,
     0,
     0,
@@ -206,9 +209,9 @@ Main arguments:\n\
                           Load/Save time of the last sample of each channel\n\
                           into a file with the same name, same directory,\n\
                           appending the suffix '%s'.\n\
-                          Allow data continuity when short disconnections occur.\n\
-                          Related to -A and -f, it enables -b.\n\
-                          DO NOT USE with -C.\n",
+                          It enables the option -b in order to allow data continuity\n\
+                          when short disconnections occur or between program restarts.\n\
+                          Related to -L, -A and -f. DO NOT USE with -C.\n",
 			  NMXP_STR_STATE_EXT
 );
 
@@ -269,6 +272,8 @@ DEFAULT_USEC / 1000, DEFAULT_N_CHANNEL, NMXP_MAX_MSCHAN_MSEC / 1000);
 
     nmxp_log(NMXP_LOG_NORM_NO, NMXP_LOG_D_ANY, "\
   -L, --listchannelsnaqs  List of the available Time Series channels on NaqsServer.\n\
+                          The output list of channels can be redirected to a file\n\
+                          which can be used as an input statefile for the option -F.\n\
   -M, --maxlatency=SECs   Max tolerable latency (default %d) [%d..%d].\n\
                           Enable NaqsServer to send out retransmission requests\n\
                           for missed packets. Inside the section NetworkInterface\n\
@@ -304,7 +309,10 @@ DAP arguments for DataServer:\n\
                           TIME is in seconds, otherwise append 'm' for minutes\n\
                           'h' for hours or 'd' for days. [1 sec .. %d days]\n\
                           DO NOT USE with -e.\n\
-  -d, --delay=TIME        Receive continuosly data with delay [%d sec .. %d days].\n\
+  -d, --delay=TIME        Receive continuosly data with a fixed delay [%d sec .. %d days].\n\
+  -a, --span_data=TIME    How much data is requested every time when receiving\n\
+                          with a fixed delay [%d sec .. %d days]. (default %ds).\n\
+                          Condition: span_data < delay - %ds. Related to -d.\n\
   -u, --username=USER     DataServer username.\n\
   -p, --password=PASS     DataServer password.\n\
   -l, --listchannels      List of the available Time Series channels on DataServer.\n\
@@ -314,7 +322,73 @@ DAP arguments for DataServer:\n\
 DEFAULT_PORT_DAP,
 (DEFAULT_INTERVAL_MAXIMUM / 86400),
 DEFAULT_DELAY_MINIMUM,
-(DEFAULT_DELAY_MAXIMUM / 86400));
+(DEFAULT_DELAY_MAXIMUM / 86400),
+DEFAULT_SPANINTERVAL_MINIMUM,
+(DEFAULT_SPANINTERVAL_MAXIMUM / 86400),
+DEFAULT_SPANINTERVAL,
+NMXP_DAP_TIMEOUT_KEEPALIVE
+    );
+
+#ifdef HAVE_LIBMSEED
+    nmxp_log(NMXP_LOG_NORM_NO, NMXP_LOG_D_ANY, "\
+Mini-SEED arguments:\n");
+
+    nmxp_log(NMXP_LOG_NORM_NO, NMXP_LOG_D_ANY, "\
+  -m, --writeseed=[TYPE]  Pack received data in Mini-SEED records and\n\
+                          store them within a SDS or BUD structure.\n\
+                          TYPE can be '%c' or '%c' (-m%c or -m%c)\n\
+                          Packets are appended to existing files. Related to -o.\n",
+			  TYPE_WRITESEED_SDS, TYPE_WRITESEED_BUD,
+			  TYPE_WRITESEED_SDS, TYPE_WRITESEED_BUD);
+    nmxp_log(NMXP_LOG_NORM_NO, NMXP_LOG_D_ANY, "\
+  -o, --outdirseed=DIR    Output directory for SDS or BUD structure.\n\
+                          Related to -m (default is current directory).\n");
+    nmxp_log(NMXP_LOG_NORM_NO, NMXP_LOG_D_ANY, "\
+  -q, --quality_indicator=IND\n\
+                          Set the quality indicator to the given character.\n\
+                          Seed 2.4 supports D (indeterminate), R (raw data),\n\
+                          Q (quality controlled data) and M (data center modified).\n\
+                          (Default is '%c').\n", DEFAULT_QUALITY_INDICATOR);
+    nmxp_log(NMXP_LOG_NORM_NO, NMXP_LOG_D_ANY, "\
+  -x, --encoding=ENC      Set the mini-SEED encoding. Either 'steim1' or 'steim2'.\n\
+                          (Default is 'steim1').\n");
+    nmxp_log(NMXP_LOG_NORM_NO, NMXP_LOG_D_ANY, "\
+  -r, --reclen            Specify the desired mini-SEED record length in bytes\n\
+                          which must be expressible as 2 raised to the power of X\n\
+                          where X is between (and including) 8 to 20.\n\
+                          (Default is %d).\n", DEFAULT_RECLEN_MINISEED);
+#endif
+
+
+#ifdef HAVE___SRC_SEEDLINK_PLUGIN_C
+    nmxp_log(NMXP_LOG_NORM_NO, NMXP_LOG_D_ANY, "\
+SeedLink arguments:\n");
+
+    nmxp_log(NMXP_LOG_NORM_NO, NMXP_LOG_D_ANY, "\
+  -k, --slink=PLUGINID    Send received data to SeedLink as a plug-in.\n\
+                          Data are sent by send_raw_depoch().\n\
+                          This option, inside the file seedlink.ini, must be\n\
+                          the last without adding value for PLUGINID!\n\
+                          PLUGINID is set by SeisComP daemon.\n\
+                          Not usable together with -K.\n");
+
+#ifdef HAVE_LIBMSEED
+    nmxp_log(NMXP_LOG_NORM_NO, NMXP_LOG_D_ANY, "\
+  -K, --slinkms=PLUGINID  Send received data to SeedLink as a plug-in.\n\
+                          This option is similar to previous -k,\n\
+                          using -K the program converts data into mini-SEED\n\
+                          records and sends them by the function send_mseed()\n\
+                          instead of send_raw_depoch().\n\
+                          Not usable together with -k.\n");
+#endif
+
+    nmxp_log(NMXP_LOG_NORM_NO, NMXP_LOG_D_ANY, "\
+  -Q, --timing_quality=TQ This value is used for the functions send_raw*().\n\
+                          TQ is %d or in [%d..%d] (default %d).\n",
+			  DEFAULT_TIMING_QUALITY, DEFAULT_TIMING_QUALITY_MINIMUM, DEFAULT_TIMING_QUALITY_MAXIMUM,
+			  DEFAULT_TIMING_QUALITY);
+
+#endif
 
     nmxp_log(NMXP_LOG_NORM_NO, NMXP_LOG_D_ANY, "\
 Other arguments:\n\
@@ -342,50 +416,9 @@ Other arguments:\n\
 	    NMXP_LOG_D_ANY
 		);
 
-#ifdef HAVE_LIBMSEED
-    nmxp_log(NMXP_LOG_NORM_NO, NMXP_LOG_D_ANY, "\
-  -m, --writeseed=[TYPE]  Pack received data in Mini-SEED records and\n\
-                          store them within a SDS or BUD structure.\n\
-                          TYPE can be '%c' or '%c' (-m%c or -m%c)\n\
-                          Packets are appended to existing files. Related to -o.\n",
-			  TYPE_WRITESEED_SDS, TYPE_WRITESEED_BUD,
-			  TYPE_WRITESEED_SDS, TYPE_WRITESEED_BUD);
-    nmxp_log(NMXP_LOG_NORM_NO, NMXP_LOG_D_ANY, "\
-  -o, --outdirseed=DIR    Output directory for SDS or BUD structure.\n\
-                          Related to -m (default is current directory).\n");
-#endif
-
     nmxp_log(NMXP_LOG_NORM_NO, NMXP_LOG_D_ANY, "\
   -w, --writefile         Dump received packets to a file.\n");
 
-#ifdef HAVE___SRC_SEEDLINK_PLUGIN_C
-    nmxp_log(NMXP_LOG_NORM_NO, NMXP_LOG_D_ANY, "\
-  -k, --slink=PLUGINID    Send received data to SeedLink as a plug-in.\n\
-                          Data are sent by send_raw_depoch().\n\
-                          This option, inside the file seedlink.ini, must be\n\
-                          the last without adding value for PLUGINID!\n\
-                          PLUGINID is set by SeisComP daemon.\n\
-                          Not usable together with -K.\n");
-#endif
-
-#ifdef HAVE_LIBMSEED
-#ifdef HAVE___SRC_SEEDLINK_PLUGIN_C
-    nmxp_log(NMXP_LOG_NORM_NO, NMXP_LOG_D_ANY, "\
-  -K, --slinkms=PLUGINID  Send received data to SeedLink as a plug-in.\n\
-                          This option is similar to previous -k,\n\
-                          using -K the program converts data into mini-SEED\n\
-                          records and sends them by the function send_mseed()\n\
-                          instead of send_raw_depoch().\n\
-                          Not usable together with -k.\n");
-#endif
-#endif
-
-    nmxp_log(NMXP_LOG_NORM_NO, NMXP_LOG_D_ANY, "\
-  -Q, --timing_quality=TQ This value is used for the functions send_raw*().\n\
-                          TQ is %d or in [%d..%d] (default %d).\n",
-			  DEFAULT_TIMING_QUALITY, DEFAULT_TIMING_QUALITY_MINIMUM, DEFAULT_TIMING_QUALITY_MAXIMUM,
-			  DEFAULT_TIMING_QUALITY);
-
 #ifndef HAVE_WINDOWS_H
     nmxp_log(NMXP_LOG_NORM_NO, NMXP_LOG_D_ANY, "\
   -E, --testport=PORT     Accept 'telnet' connection on PORT\n\
@@ -403,6 +436,9 @@ Signal handling:\n\
                           to immediately attempt to gracefully terminate.\n\
                           It may take several seconds to complete exiting.\n\
    ALRM                   Print current info about Raw Stream buffer.\n\
+   USR1                   Force to close a connection and open again\n\
+                          without quitting the program. Only for connection\n\
+                          in near real-time to NaqsServer.\n\
    HUP PIPE               Ignored. (SIG_IGN)\n\
 \n", NMXP_LOG_STR(PACKAGE_NAME));
 
@@ -430,8 +466,10 @@ char *get_channel_list_argument_from_state_file(const char *filename) {
     char *ret_channel_string = NULL;
     char line[MAXSIZE_LINE_CHAN_STATE];
     char str_chan[MAXSIZE_CHANNEL_STRING];
-    int k;
+    int k, c;
     FILE *fstatefile = NULL;
+    int i;
+    int flag_insert = 1;
 
     fstatefile = fopen(filename, "r");
 
@@ -441,24 +479,62 @@ char *get_channel_list_argument_from_state_file(const char *filename) {
 	ret_channel_string[0] = 0;
 	while(fgets(line, MAXSIZE_LINE_CHAN_STATE, fstatefile) != NULL) {
 	    k = 0;
+	    c = 0;
+
+	    /* eat any white spaces and tabs at the beginning */
 	    while(line[k] != 0
-		    &&  line[k] != ' '
+		    &&  (line[k] == ' ' ||  line[k] == '\t')
+		    &&  k < MAXSIZE_CHANNEL_STRING) {
+		k++;
+	    }
+
+	    /* copy any character up to the end of the line or # character */
+	    while(line[k] != 0
+		    &&  line[k] != '#'
 		    &&  line[k] != 10
 		    &&  line[k] != 13
 		    &&  k < MAXSIZE_CHANNEL_STRING) {
-		str_chan[k] = line[k];
+		str_chan[c] = line[k];
 		k++;
+		c++;
 	    }
-	    str_chan[k] = 0;
-	    if(ret_channel_string[0] == 0) {
-		strncpy(ret_channel_string, str_chan, MAXSIZECHANNELSTRINGARGUMENT);
+	    str_chan[c] = 0;
+
+	    /* remove possible white spaces and tabs from the end of the str_chan */
+	    i = strlen(str_chan);
+	    while(i > 0) {
+		    if(str_chan[i] == ' ' || str_chan[i] == '\t') {
+			str_chan[i] = 0;
+		    }
+		    i--;
+	    }
+
+	    /* check if str_chan is blank, before insert into the channel list */
+	    flag_insert = 0;
+	    i = 0;
+	    while(i < strlen(str_chan)  &&  !flag_insert) {
+		if(str_chan[i] != ' ') {
+		    flag_insert = 1;
+		}
+		i++;
+	    }
+
+	    if(flag_insert) {
+		if(ret_channel_string[0] == 0) {
+		    strncpy(ret_channel_string, str_chan, MAXSIZECHANNELSTRINGARGUMENT);
+		} else {
+		    strncat(ret_channel_string, ",", MAXSIZECHANNELSTRINGARGUMENT - strlen(ret_channel_string));
+		    strncat(ret_channel_string, str_chan, MAXSIZECHANNELSTRINGARGUMENT - strlen(ret_channel_string));
+		}
 	    } else {
-		strncat(ret_channel_string, ",", MAXSIZECHANNELSTRINGARGUMENT);
-		strncat(ret_channel_string, str_chan, MAXSIZECHANNELSTRINGARGUMENT);
+		nmxp_log(NMXP_LOG_NORM, NMXP_LOG_D_CHANSTATE, "Discarded '%s' from station file %s:\n%s",
+			NMXP_LOG_STR(str_chan), NMXP_LOG_STR(filename), NMXP_LOG_STR(line));
 	    }
 	}
 	fclose(fstatefile);
     }
+    nmxp_log(NMXP_LOG_NORM_NO, NMXP_LOG_D_CHANSTATE, "Channel list: %s\n",
+	    NMXP_LOG_STR(ret_channel_string));
     return ret_channel_string;
 }
 
@@ -528,6 +604,11 @@ int nmxptool_getopt_long(int argc, char **argv, NMXPTOOL_PARAMS *params)
     char one_time_option[255];
     int c;
 
+#ifdef HAVE_LIBMSEED
+    int flag_reclen_pow = 0;
+    int reclen_pow = DEFAULT_RECLEN_MINIMUM;
+#endif
+
     /*
     int len_int, j;
     char unit = 'X';
@@ -556,6 +637,7 @@ int nmxptool_getopt_long(int argc, char **argv, NMXPTOOL_PARAMS *params)
 	{"end_time",     required_argument, NULL, 'e'},
 	{"interval",     required_argument, NULL, 't'},
 	{"delay",        required_argument, NULL, 'd'},
+	{"span_data",    required_argument, NULL, 'a'},
 	{"username",     required_argument, NULL, 'u'},
 	{"password",     required_argument, NULL, 'p'},
 	{"maxlatency",   required_argument, NULL, 'M'},
@@ -573,6 +655,9 @@ int nmxptool_getopt_long(int argc, char **argv, NMXPTOOL_PARAMS *params)
 #ifdef HAVE_LIBMSEED
 	{"writeseed",    required_argument, NULL, 'm'},
 	{"outdirseed",   required_argument, NULL, 'o'},
+	{"quality_indicator", required_argument, NULL, 'q'},
+	{"encoding",     required_argument, NULL, 'x'},
+	{"reclen",       required_argument, NULL, 'r'},
 #endif
 	{"writefile",    no_argument,       NULL, 'w'},
 #ifdef HAVE___SRC_SEEDLINK_PLUGIN_C
@@ -583,7 +668,9 @@ int nmxptool_getopt_long(int argc, char **argv, NMXPTOOL_PARAMS *params)
 	{"slinkms",      required_argument, NULL, 'K'},
 #endif
 #endif
+#ifdef HAVE___SRC_SEEDLINK_PLUGIN_C
 	{"timing_quality", required_argument, NULL, 'Q'},
+#endif
 #ifndef HAVE_WINDOWS_H
 	{"socketport",   required_argument, NULL, 'E'},
 #endif
@@ -594,7 +681,7 @@ int nmxptool_getopt_long(int argc, char **argv, NMXPTOOL_PARAMS *params)
 	{0, 0, 0, 0}
     };
 
-    char optstr[300] = "H:P:D:C:N:n:S:R:s:e:t:d:u:p:M:T:v:B:A:F:f:gGblLiwhV";
+    char optstr[300] = "H:P:D:C:N:n:S:R:s:e:t:d:a:u:p:M:T:v:B:A:F:f:gGblLiwhV";
 
     int option_index = 0;
 
@@ -602,11 +689,14 @@ int nmxptool_getopt_long(int argc, char **argv, NMXPTOOL_PARAMS *params)
 #ifdef HAVE_LIBMSEED
     strcat(optstr, "m:");
     strcat(optstr, "o:");
+    strcat(optstr, "q:");
+    strcat(optstr, "x:");
+    strcat(optstr, "r:");
 #endif
 
-    strcat(optstr, "Q:");
 
 #ifdef HAVE___SRC_SEEDLINK_PLUGIN_C
+    strcat(optstr, "Q:");
     strcat(optstr, "k:");
 #endif
 
@@ -704,6 +794,9 @@ int nmxptool_getopt_long(int argc, char **argv, NMXPTOOL_PARAMS *params)
 		case 'S':
 		    params->stc = atoi(optarg);
 		    nmxp_log(NMXP_LOG_NORM, NMXP_LOG_D_ANY, "Short-Term-Completion %d.\n", params->stc);
+		    if(params->stc >= 0) {
+			params->rate = 0; // original sample rate
+		    }
 		    break;
 
 		case 'R':
@@ -736,6 +829,10 @@ int nmxptool_getopt_long(int argc, char **argv, NMXPTOOL_PARAMS *params)
 		    ret_errors += nmxptool_read_time(optarg, &(params->delay) );
 		    break;
 
+		case 'a':
+		    ret_errors += nmxptool_read_time(optarg, &(params->span_data) );
+		    break;
+
 		case 'u':
 		    params->datas_username = optarg;
 		    break;
@@ -810,9 +907,12 @@ int nmxptool_getopt_long(int argc, char **argv, NMXPTOOL_PARAMS *params)
 		    break;
 #endif
 #endif
+
+#ifdef HAVE___SRC_SEEDLINK_PLUGIN_C
 		case 'Q':
 		    params->timing_quality = atoi(optarg);
 		    break;
+#endif
 
 #ifndef HAVE_WINDOWS_H
 		case 'E':
@@ -821,7 +921,6 @@ int nmxptool_getopt_long(int argc, char **argv, NMXPTOOL_PARAMS *params)
 #endif
 
 		case 'F':
-		    params->flag_buffered = 1;
 		    params->statefile = optarg;
 		    if(params->channels == NULL) {
 			params->channels = get_channel_list_argument_from_state_file(params->statefile);
@@ -896,6 +995,7 @@ int nmxptool_getopt_long(int argc, char **argv, NMXPTOOL_PARAMS *params)
 				"Syntax error in option -%c %s!\n", c, NMXP_LOG_STR(optarg));
 		    }
 		    break;
+
 		case 'o':
 		    params->outdirseed = optarg;
 		    if(params->outdirseed) {
@@ -904,10 +1004,60 @@ int nmxptool_getopt_long(int argc, char **argv, NMXPTOOL_PARAMS *params)
 			} else {
 			    ret_errors++;
 			    nmxp_log(NMXP_LOG_NORM_NO, NMXP_LOG_D_ANY,
-				    "Mini-SEED output directory %s does not exists!\n", NMXP_LOG_STR(params->outdirseed));
+				    "Mini-SEED output directory %s does not exist!\n", NMXP_LOG_STR(params->outdirseed));
 			}
 		    }
 		    break;
+
+                case 'q':
+                    switch(optarg[0]) {
+                      case 'D':
+                      case 'R':
+                      case 'Q':
+                      case 'M':
+                        params->quality_indicator = optarg[0];
+                        break;
+                      default:
+                        ret_errors++;
+                        nmxp_log(NMXP_LOG_NORM_NO, NMXP_LOG_D_ANY,
+                               "Quality indicator %s is invalid! Must be D, R, Q or M!\n", NMXP_LOG_STR(optarg));
+                        break;
+                    }
+		    break;
+
+		case 'x':
+		    if (strcmp(optarg,"steim1") == 0) {
+			params->encoding = DE_STEIM1;
+		    } else if (strcmp(optarg,"steim2") == 0) {
+			params->encoding = DE_STEIM2;
+		    } else {
+			ret_errors++;
+			nmxp_log(NMXP_LOG_NORM_NO, NMXP_LOG_D_ANY,
+				"Encoding %s is invalid! Must either be steim1 or steim2!\n",NMXP_LOG_STR(optarg));
+		    }
+		    break;
+
+		case 'r':
+		    params->reclen = atoi(optarg);
+		    if(params->reclen >= DEFAULT_RECLEN_MINIMUM  &&  params->reclen <= DEFAULT_RECLEN_MAXIMUM) {
+			flag_reclen_pow = 0;
+			while(!flag_reclen_pow  &&  reclen_pow <= DEFAULT_RECLEN_MAXIMUM) {
+			    if(params->reclen == reclen_pow) {
+				flag_reclen_pow = 1;
+			    }
+			    reclen_pow *= 2;
+			}
+			if(!flag_reclen_pow) {
+			    ret_errors++;
+			    nmxp_log(NMXP_LOG_NORM_NO, NMXP_LOG_D_ANY,
+				    "reclen must expressible as 2 raised to the power of X where X is between (and including) 8 to 20.\n");
+			}
+		    } else {
+			ret_errors++;
+			nmxp_log(NMXP_LOG_NORM_NO, NMXP_LOG_D_ANY,
+				"reclen must expressible as 2 raised to the power of X where X is between (and including) 8 to 20.\n");
+		    }
+		    break;
 #endif
 
 		case 'w':
@@ -951,6 +1101,12 @@ int nmxptool_getopt_long(int argc, char **argv, NMXPTOOL_PARAMS *params)
 	putchar ('\n');
     }
 
+    if(params->statefile) {
+	params->flag_buffered = 1;
+	nmxp_log(NMXP_LOG_NORM_NO, NMXP_LOG_D_ANY,
+		"Enable buffer for requesting also recent packets into the past.\n");
+    }
+
     return ret_errors;
 }
 
@@ -1106,6 +1262,15 @@ int nmxptool_check_params(NMXPTOOL_PARAMS *params) {
 	ret = -1;
 	nmxp_log(NMXP_LOG_NORM_NO, NMXP_LOG_D_ANY, "<delay> has to be in the interval [%d..%d] secs.\n",
 		DEFAULT_DELAY_MINIMUM, DEFAULT_DELAY_MAXIMUM);
+    } else if(params->span_data < DEFAULT_SPANINTERVAL_MINIMUM || params->span_data > DEFAULT_SPANINTERVAL_MAXIMUM) {
+	ret = -1;
+	nmxp_log(NMXP_LOG_NORM_NO, NMXP_LOG_D_ANY, "<span_data> has to be in the interval [%d..%d] secs.\n",
+		DEFAULT_SPANINTERVAL_MINIMUM, DEFAULT_SPANINTERVAL_MAXIMUM);
+    } else if(params->delay != DEFAULT_DELAY &&
+	    (params->span_data >= params->delay - NMXP_DAP_TIMEOUT_KEEPALIVE)) {
+	ret = -1;
+	nmxp_log(NMXP_LOG_NORM_NO, NMXP_LOG_D_ANY, "<span_data> has to be less than delay - %ds. Current value should be less than %ds.\n",
+		NMXP_DAP_TIMEOUT_KEEPALIVE, params->delay - NMXP_DAP_TIMEOUT_KEEPALIVE);
     } else if(params->verbose_level < DEFAULT_VERBOSE_LEVEL_MINIMUM  ||  params->verbose_level > DEFAULT_VERBOSE_LEVEL_MAXIMUM) {
 	ret = -1;
 	nmxp_log(NMXP_LOG_NORM_NO, NMXP_LOG_D_ANY, "<verbose_level> has to be in the interval [%d..%d].\n",
@@ -1174,12 +1339,14 @@ int nmxptool_check_params(NMXPTOOL_PARAMS *params) {
 	nmxp_log(NMXP_LOG_WARN, NMXP_LOG_D_ANY, "<timeoutrecv> ignored since not defined --stc=-1.\n");
     }
 
+#ifdef HAVE___SRC_SEEDLINK_PLUGIN_C
     if( params->timing_quality != DEFAULT_TIMING_QUALITY &&
 	    ( params->timing_quality < DEFAULT_TIMING_QUALITY_MINIMUM ||  params->timing_quality > DEFAULT_TIMING_QUALITY_MAXIMUM) ) {
 	ret = -1;
 	nmxp_log(NMXP_LOG_NORM_NO, NMXP_LOG_D_ANY, "<timing_quality> could be %d or within [%d .. %d].\n",
 		DEFAULT_TIMING_QUALITY, DEFAULT_TIMING_QUALITY_MINIMUM, DEFAULT_TIMING_QUALITY_MAXIMUM);
     }
+#endif
 
     if(params->usec == 0  &&  params->n_channel == 0) {
 	/* Do nothing */
