@@ -1037,7 +1037,7 @@ WS_ERR_NO_CONNECTION: if a connection could not be established
     /* An error occured;
     * don't blab about here since we already did earlier. */
 Abort:
-    menu->sock = -1; /* mark the socket as dead */
+    if (menu) menu->sock = -1; /* mark the socket as dead */
     return ret;
 }
 
@@ -2179,10 +2179,10 @@ int wsSearchSCNL( TRACE_REQ* getThis, WS_MENU* menup, wsHANDLE pEnv)
 
   
   memset(&PSCNL, 0, sizeof(WS_PSCNL_REC));
-  strncpy(PSCNL.sta,  getThis->sta,  sizeof(PSCNL.sta - 1));
-  strncpy(PSCNL.chan, getThis->chan, sizeof(PSCNL.chan - 1));
-  strncpy(PSCNL.net,  getThis->net,  sizeof(PSCNL.net - 1));
-  strncpy(PSCNL.loc,  getThis->loc,  sizeof(PSCNL.loc - 1));
+  strncpy(PSCNL.sta,  getThis->sta,  sizeof(PSCNL.sta) - 1);
+  strncpy(PSCNL.chan, getThis->chan, sizeof(PSCNL.chan) - 1);
+  strncpy(PSCNL.net,  getThis->net,  sizeof(PSCNL.net) - 1);
+  strncpy(PSCNL.loc,  getThis->loc,  sizeof(PSCNL.loc) - 1);
 
 
   /* find the PSCNL */
